import { prefixes as knownPrefixes } from '@zazuko/rdf-vocabularies';
import { SerializationStrategy, TemplateResult } from './TemplateResult';
import * as turtleSyntax from './syntax/turtle';
function prefixDeclarations(prefixes) {
    return [...prefixes]
        .filter(prefix => prefix in knownPrefixes)
        .map(prefix => `PREFIX ${prefix}: <${knownPrefixes[prefix]}>`);
}
export class SparqlStrategy extends SerializationStrategy {
    evaluateLiteral(term, options) {
        return turtleSyntax.literal(term, options.base);
    }
    evaluateNamedNode(term, options) {
        return turtleSyntax.namedNode(term, options.base);
    }
    evaluateBlankNode(term) {
        return {
            value: turtleSyntax.blankNode(term),
            prefixes: [],
        };
    }
    evaluateVariable(term) {
        return {
            value: `?${term.value}`,
            prefixes: [],
        };
    }
    getFinalString(result, prefixes, options) {
        const prologue = options.prologue || typeof options.prologue === 'undefined';
        let prologueLines = [];
        if (prologue) {
            prologueLines = prefixDeclarations(prefixes);
            if (options.base) {
                prologueLines = [`BASE <${options.base}>`, ...prologueLines];
            }
            if (prologueLines.length > 0) {
                prologueLines.push('\n');
            }
        }
        return `${prologueLines.join('\n')}${result}`;
    }
    evaluateDataset() {
        throw new Error('Method not implemented');
    }
    evaluateQuad() {
        throw new Error('Method not implemented');
    }
}
export const sparql = (strings, ...values) => new TemplateResult({
    strings,
    values,
    tag: sparql,
    strategy: new SparqlStrategy(),
    defaultOptions: {
        prologue: true,
    },
});
